#!/usr/bin/env escript
%%
%%! +P 524288

-module(tfib).
-export([main/1]).
-mode(compile).
-compile({nowarn_unused_function, [short_delay_times/1, time/1]}).

busy_sleep(N) ->
    if
        N > 0 -> busy_sleep(N-1);
        true -> N
    end.

short_delay() ->
    busy_sleep(7700). % 50 us delay

short_delay_times(N) ->
    if
        N > 0 -> short_delay(), short_delay_times(N-1);
        true -> N
    end.

tfib(N) ->
    short_delay(),
    if
        N < 2 ->
            1;
        true ->
            Pid = self(),
            spawn(fun() -> Pid ! tfib(N-2) end),
            Y = tfib(N-1),
            receive
                X -> X + Y
            end
    end.

range(I, J) -> lists:seq(I, J-1).

go_join(0, Lst) -> Lst;
go_join(N, Lst) ->
    receive
        X -> go_join(N-1, [X|Lst])
    end.

go(N, Repeat) ->
    Pid = self(),
    [spawn(fun() -> Pid ! tfib(N) end) || _ <- range(0, Repeat)],
    go_join(Repeat, []).

time(F) ->
    B = erlang:system_time(millisecond),
    F(),
    A = erlang:system_time(millisecond),
    A-B.

main(_) ->
%    io:format("short_delay = ~w us~n",[time(fun() -> short_delay_times(1000) end)]),
    go(15, 500).
